'' =================================================================================================
''
''   File....... jm_tslx6_monitor_v2.spin
''   Purpose.... * selectively enable non-contiguous sensor channels
''   Author..... Jon "JonnyMac" McPhalen
''               -- sensor code by Andy "Ariba" Schenk
''               -- reformatted by JM
''   E-mail..... 
''   Started....
''   Updated.... 21 FEB 2018
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  TRM_BAUD = 115_200                                            ' terminal baud rate
  

con { io pins }                                                  
                                                                 
  RX1  = 31  { I }                                              ' serial / programming
  TX1  = 30  { O }                                            
                                                                 
  SDA1 = 29  { I/O }                                            ' i2c / eeprom
  SCL1 = 28  { I/O } 


  CH5  = 27  { I }                                              ' tls sensor pins   
  CH4  =  4  { I }
  CH3  =  3  { I }
  CH2  =  2  { I }  
  CH1  =  1  { I }
  CH0  = 26  { I }                                                      
  

obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io"                                                '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  byte  hzstr[11]                                                             
                                                                 
                                                                 
pub main : n | t, idx                                                        
                                                                 
  setup                                                          
                                                                 
  term.rxflush                                                  ' wait open terminal & keypress
  term.rx                                                        
  term.tx(term#CLS)


' ---------------------------------------------------------------------------------------
  
  ' test frquencies for PAB
  ' -- frqx = hz * (2^32 / CLK_FREQ)

  ctra := (%00100 << 26) + 26         
  frqa := 662799                                                ' 12345.6 Hz on P26
                       
  ctrb := (%00100 << 26) + 27
  frqb := 5368709                                               ' 100 kHz on P27 
                                                                
  dira[27..26] := %11                                           ' enable outputs

' ---------------------------------------------------------------------------------------


  ' sensor monitor / display loop                                             

  t := cnt
  repeat
    waitcnt(t += CLK_FREQ)
    term.dec(++n)
    term.tx(term#TAB)
    repeat idx from 0 to 5
      term.str(tix2hz(tsl[idx], @hzstr))
      if (idx < 5)
        term.str(string(", "))
      else
        term.tx(13)
      

con

  TENTHS = CLK_FREQ * 10                                        ' for 0.1Hz resolution
  

pub tix2hz(tix, p_str) : idx | hz, divisor, zflag, digit

  bytefill(p_str, 0, 11)                                        ' nnnnnnnn.n + NUL

  hz := TENTHS / tix
  divisor := 10000000_0 
  zflag := 0                                                    ' suppress leading zeroes

  repeat while (divisor)
    digit := hz / divisor                                       ' get this digit                                         
    zflag += digit                                              ' update zflag (>0 = print digit)

    if (zflag) or (divisor =< 1_0)                              ' print digit?
      byte[p_str][idx++] := "0" + digit
    else
      byte[p_str][idx++] := " "                                 ' no, use leading space

    hz //= divisor                                              ' remove this digit
    divisor /= 10                                               ' update divisor

    if (divisor == 0_1)
      byte[p_str][idx++] := "."                                 ' decimal pt before last digit

  return p_str  

                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal

  start_sensors                                                 ' start sensor cog
                                                                 

con

  { ----------------------- }
  {  T L S   M O N I T O R  }
  { ----------------------- }


  S_ENABLE = %100001                                            ' enable CH0 and CH5 
  

var

  long  masks[6]                                                ' input masks
  long  tsl[6]                                                  ' sensor ticks / period


pub start_sensors | idx

  masks[0] := 1 << CH0                                          ' convert pin# to mask
  masks[1] := 1 << CH1
  masks[2] := 1 << CH2
  masks[3] := 1 << CH3
  masks[4] := 1 << CH4
  masks[5] := 1 << CH5

  repeat idx from 0 to 5                                        ' loop through sensors
    ifnot (S_ENABLE & |<idx)                                    ' if not enabled
      masks[idx] := 0                                           '  clear the mask

  longfill(@tsl, 0, 6)                                          ' clear period array

  cognew(@tsl6x, @masks)                                        ' start monitor cog 
  

dat

                        org     0

tsl6x                   mov     sensors, #6                     ' scan six sensors
                        mov     hub, par                        ' get address of masks in hub

loop                    rdlong  chmask, hub                     ' read channel mask
                        tjz     chmask, #skipit                 ' skip if no mask

                        waitpne chmask, chmask                  ' wait for low
                        waitpeq chmask, chmask                  ' wait for high
                        mov     t1, cnt                         ' mark start of cycle
                        
                        waitpne chmask, chmask                  ' wait for low   
                        waitpeq chmask, chmask                  ' wait for high
                        mov     t2, cnt                         ' mark end of cycle
                        sub     t2, t1                          ' calc ticks in cycle

                        add     hub, #6*4                       ' skip over masks
                        wrlong  t2, hub                         ' write ticks to tsl[]
                        sub     hub, #6*4                       ' back to masks                         

skipit                  add     hub, #4                         ' point to next
                        djnz    sensors, #loop                       

                        jmp     #tsl6x

sensors                 res     1                               ' # of sensors
hub                     res     1                               ' pointer to hub
chmask                  res     1                               ' current channel mask
t1                      res     1                               ' start of cycle
t2                      res     1                               ' end of cycle

                        fit     496
                        
                             
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      